using System;
using System.Text;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;

namespace Lexer
{

    public class LexerException : System.Exception
    {
        public LexerException(string msg)
            : base(msg)
        {
        }

    }

    public class Lexer
    {

        protected int position;
        protected char currentCh; //   
        protected int currentCharValue; //     
        protected System.IO.StringReader inputReader;
        protected string inputString;

        public Lexer(string input)
        {
            inputReader = new System.IO.StringReader(input);
            inputString = input;
        }

        public void Error()
        {
            System.Text.StringBuilder o = new System.Text.StringBuilder();
            o.Append(inputString + '\n');
            o.Append(new System.String(' ', position - 1) + "^\n");
            o.AppendFormat("Error in symbol {0}", currentCh);
            throw new LexerException(o.ToString());
        }

        protected void NextCh()
        {
            this.currentCharValue = this.inputReader.Read();
            this.currentCh = (char) currentCharValue;
            this.position += 1;
        }

        public virtual bool Parse()
        {
            return true;
        }
    }

    public class IntLexer : Lexer
    {

        protected System.Text.StringBuilder intString;
        public int parseResult = 0;

        public IntLexer(string input)
            : base(input)
        {
            intString = new System.Text.StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            int sig = 1;
            if (currentCh == '+')
            {

                NextCh();
            }
            else if (currentCh == '-')
            {
                sig = -1;
                NextCh();
            }
        
            if (char.IsDigit(currentCh))
            {
                parseResult += (currentCh - '0');
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh))
            { 
                parseResult = (parseResult * 10) + (currentCh - '0');
                NextCh();
            }


            if (currentCharValue != -1)
            {
                Error();
            }

            parseResult *= sig;

            return true;

        }
    }
    
    public class IdentLexer : Lexer
    {
        private string parseResult;
        protected StringBuilder builder;
    
        public string ParseResult
        {
            get { return parseResult; }
        }
    
        public IdentLexer(string input) : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();

            if(char.IsLetter(currentCh) || currentCh == '_')
            {
                parseResult += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh) || char.IsLetter(currentCh) || currentCh == '_')
            {
                parseResult += currentCh;
                NextCh();
            }


            if (currentCharValue != -1)
            {
                Error();
            }

            if(parseResult == null)
            {
                Error();
            }

            return true;
        }
       
    }

    public class IntNoZeroLexer : IntLexer
    {
        public IntNoZeroLexer(string input)
            : base(input)
        {
        }

        public override bool Parse()
        {
            NextCh();
            int sig = 1;
            if (currentCh == '+')
            {

                NextCh();
            }
            else if (currentCh == '-')
            {
                sig = -1;
                NextCh();
            }

            if (char.IsDigit(currentCh) && currentCh != '0')
            {
                parseResult += (currentCh - '0');
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh))
            {
                parseResult = (parseResult * 10) + (currentCh - '0');
                NextCh();
            }


            if (currentCharValue != -1)
            {
                Error();
            }

            parseResult *= sig;

            return true;

        
        }
    }

    public class LetterDigitLexer : Lexer
    {
        protected StringBuilder builder;
        protected string parseResult;

        public string ParseResult
        {
            get { return parseResult; }
        }

        public LetterDigitLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();

            bool isNumberExpected = false; 


            if (char.IsLetter(currentCh))
            {
                parseResult += currentCh;
                isNumberExpected = true; 
                NextCh();
            }
            else 
            {
                Error();
            }

            while (char.IsDigit(currentCh) || char.IsLetter(currentCh))
            {
                if (isNumberExpected && char.IsDigit(currentCh))
                {
                    parseResult += currentCh;
                    isNumberExpected = false;
                    NextCh();
                }
                else if (!isNumberExpected && char.IsLetter(currentCh))
                {
                    parseResult += currentCh;
                    isNumberExpected = true;
                    NextCh();
                }
                else
                {
                    Error();
                }

            }

            if (currentCharValue != -1)
            {
                Error();
            }

            if (parseResult == null)
            {
                Error();
            }

            return true;
        }
       
    }

    public class LetterListLexer : Lexer
    {
        protected List<char> parseResult;

        public List<char> ParseResult
        {
            get { return parseResult; }
        }

        public LetterListLexer(string input)
            : base(input)
        {
            parseResult = new List<char>();
        }

        public override bool Parse()
        {
            string s = "";
            NextCh();
            bool isSeparatorExpected = false;

            if (char.IsLetter(currentCh))
            {
                s += currentCh;
                isSeparatorExpected = true;
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsLetter(currentCh) || currentCh == ',' || currentCh == ';')
            {
                if (isSeparatorExpected && (currentCh == ',' || currentCh == ';'))
                {
                    s += currentCh;
                    isSeparatorExpected = false;
                    NextCh();
                }
                else if (!isSeparatorExpected && char.IsLetter(currentCh))
                {
                    s += currentCh;
                    isSeparatorExpected = true;
                    NextCh();
                }
                else
                {
                    Error();
                }
            }

            
            if (isSeparatorExpected)
            {
                var arr = s.Split(new Char[] { ',', ';' });
                foreach (string ch in arr)
                {
                    parseResult.Add(ch.ToCharArray()[0]);
                }
            }
            

            if (currentCharValue != -1)
            {
                Error();
            }

            if (parseResult.Count == 0)
            {
                Error();
            }

            return true;
        }
    }

    public class DigitListLexer : Lexer
    {
        protected List<int> parseResult;

        public List<int> ParseResult
        {
            get { return parseResult; }
        }

        public DigitListLexer(string input)
            : base(input)
        {
            parseResult = new List<int>();
        }

        public override bool Parse()
        {
            string s = "";
            NextCh();

            if (char.IsDigit(currentCh))
            {
                s += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh) || char.IsWhiteSpace(currentCh))
            {
                s += currentCh;
                NextCh();
            }


            if (!char.IsWhiteSpace(s[s.Length-1]))
            {
                var arr = System.Text.RegularExpressions.Regex.Split(s, @"\s+");
                if(arr.Length <= 0)
                {
                    Error();
                }
                foreach (string ch in arr)
                {
                    if(ch.Length >= 2)
                    {
                        Error();
                    }
                    parseResult.Add(ch.ToCharArray()[0] - '0');
                }
            }
            else
            {
                Error();
            }


            if (currentCharValue != -1)
            {
                Error();
            }

            if (parseResult.Count == 0)
            {
                Error();
            }

            return true;
        }
    }

    public class LetterDigitGroupLexer : Lexer
    {
        protected StringBuilder builder;
        protected string parseResult;

        public string ParseResult
        {
            get { return parseResult; }
        }
        
        public LetterDigitGroupLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            int countLetters = 0;
            int countDigits = 0;
            NextCh();

            if(char.IsLetter(currentCh))
            {
                parseResult += currentCh;
                countLetters++;
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh) || char.IsLetter(currentCh))
            {
                if (char.IsDigit(currentCh))
                {
                    countLetters = 0;
                    if (countDigits < 2)
                    {
                        parseResult += currentCh;
                        countDigits++;
                        NextCh();
                    }
                    else
                    {
                        Error();
                    }
                }
                else
                {
                    countDigits = 0;
                    if (countLetters < 2)
                    {
                        parseResult += currentCh;
                        countLetters++;
                        NextCh();
                    }
                    else
                    {
                        Error();
                    }

                }
            }

            if (currentCharValue != -1)
            {
                Error();
            }

            if (parseResult == null)
            {
                Error();
            }

            return true;
        }
       
    }

    public class DoubleLexer : Lexer
    {
        private StringBuilder builder;
        private double parseResult;

        public double ParseResult
        {
            get { return parseResult; }

        }

        public DoubleLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            
            bool dotExpected = true;

            if (char.IsDigit(currentCh))
            {
                parseResult += (currentCh - '0');
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh) || currentCh == '.')
            {
                if (currentCh == '.' && !dotExpected)
                {
                    Error();
                }
                else if (currentCh == '.' && dotExpected)
                {    
                    dotExpected = false;
                    NextCh();
                    if(currentCharValue == -1)
                    {
                        Error();
                    }
                }

                if (char.IsDigit(currentCh) && dotExpected)
                {
                    parseResult = (parseResult * 10) + (currentCh - '0');
                    NextCh();
                }
                else if (char.IsDigit(currentCh) && !dotExpected)
                {
                    parseResult = ((parseResult * 10) + (currentCh - '0')) / 10;
                    NextCh();
                }

            }
            if (currentCharValue != -1)
            {
                Error();
            }

            return true;
        }
       
    }

    public class StringLexer : Lexer
    {
        private StringBuilder builder;
        private string parseResult;

        public string ParseResult
        {
            get { return parseResult; }

        }

        public StringLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {

            NextCh();

            if (currentCh == '\'')
            {
                parseResult += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }

            while (currentCh != '\'')
            {
                if (currentCharValue == -1)
                {
                    Error();
                }
                else
                {
                    parseResult += currentCh;
                    NextCh();
                }
            }

            parseResult += currentCh;
            NextCh();

            if (currentCharValue != -1)
            {
                Error();
            }


            if (parseResult == null)
            {
                Error();
            }

            return true;
        }
    }

    public class CommentLexer : Lexer
    {
        private StringBuilder builder;
        private string parseResult;

        public string ParseResult
        {
            get { return parseResult; }

        }

        public CommentLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();

            if (currentCh == '/')
            {
                parseResult += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }
            
            if (currentCh == '*')
            {
                parseResult += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }

            while (currentCh != '/')
            {
                if (currentCharValue == -1)
                {
                    Error();
                }
               
                while(currentCh != '*')
                {
                    if (currentCharValue == -1)
                    {
                        Error();
                    }

                    parseResult += currentCh;
                    NextCh();
                }

                parseResult += currentCh;
                NextCh();
            }

            parseResult += currentCh;
            NextCh();

            if (currentCharValue != -1)
            {
                Error();
            }


            if (parseResult == null)
            {
                Error();
            }

            return true;
        }
    }

    public class IdentChainLexer : Lexer
    {
        private StringBuilder builder;
        private List<string> parseResult;

        public List<string> ParseResult
        {
            get { return parseResult; }

        }

        public IdentChainLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
            parseResult = new List<string>();
        }

        public override bool Parse()
        {
            string s = "";
            NextCh();
            bool areSymbolsAllowed = false;

            if (char.IsLetter(currentCh))
            {
                s += currentCh;
                areSymbolsAllowed = true;
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsLetter(currentCh) || char.IsNumber(currentCh)|| currentCh == '.' || currentCh == '_')
            {
                if (!areSymbolsAllowed && (currentCh == '_' || char.IsNumber(currentCh)))
                {
                    Error();
                }
                else if (currentCh == '.')
                {
                    s += currentCh;
                    areSymbolsAllowed = false;
                    NextCh();
                }
                else
                {
                    s += currentCh;
                    areSymbolsAllowed = true;
                    NextCh();
                }
            }

            if(areSymbolsAllowed)
            {
                var arr = s.Split('.');
                foreach (string str in arr)
                {
                    parseResult.Add(str);
                }
            }
            else
            {
                Error();
            }
        

            if (currentCharValue != -1)
            {
                Error();
            }

            if (parseResult.Count == 0)
            {
                Error();
            }

            return true;
        
        }
    }

    public class Program
    {
        public static void Main()
        {
            string input = "154216";
            Lexer L = new IntLexer(input);
            try
            {
                L.Parse();
            }
            catch (LexerException e)
            {
                System.Console.WriteLine(e.Message);
            }

        }
    }
}